/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 7
 * Práctica 1
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <cwiid.h>

cwiid_mesg_callback_t cwiid_callback; //Función para callback
cwiid_err_t error; //Función para errores

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

//Función de callback
void cwiid_callback(cwiid_wiimote_t *wiimote, int mesg_count, union cwiid_mesg mesg[], struct timespec *timestamp) {
	int i;

	//Datos iniciales de la llamada al callback
	printf("Callback, mesg_count: %d\n", mesg_count);
	printf("Time: %li (sec), %li (nsec)\n", timestamp->tv_sec, timestamp->tv_nsec);
	printf("----------------------------------------\n");
	//Bucle que recorre todos los mensajes incluidos en la llamada al callback
	for (i = 0; i < mesg_count; i++) {
		printf("Mensaje nº %i, tipo: %i\n", i, mesg[i].type);
		printf("Estado-> tipo: %i, batería: %i, accesorio: %i\n", mesg[i].status_mesg.type, (int)(100.0*mesg[i].status_mesg.battery/CWIID_BATTERY_MAX), mesg[i].status_mesg.ext_type);
		printf("----------------------------------------\n");
	}
	printf("##############################################################\n");
}


int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del Wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del Wiimote
	int salir = 0; //Variable para interrumpir la ejecución
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el Wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier Wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, CWIID_FLAG_MESG_IFC))) { //Conexión y habilitación de envío de mensajes
		printf("No se pudo detectar Wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del Wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		if (cwiid_set_mesg_callback(wiimote, cwiid_callback)) { //Establezco el callback
			printf("No se pudo establecer el callback\n");
		}
		else {
			if (cwiid_set_rpt_mode(wiimote, CWIID_RPT_STATUS)) {
				printf("Error al establecer modo de informe\n");
			}
			else {
				printf("Escribe x y pulsa enter para salir\n");
				while (!salir) {
					//Bucle principal, sólo espera
					if (getchar() == 'x') {salir = 1;}
				}
			}
		}
		if (cwiid_disconnect(wiimote)) { //Desconexión del Wiimote
			printf("Error al desconectar el Wiimote\n");
		}
	}
	return 0;
}
