/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 7
 * Práctica 4
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <cwiid.h>

cwiid_mesg_callback_t cwiid_callback; //Función para callback
cwiid_err_t error; //Función para errores

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

//Función de callback
void cwiid_callback(cwiid_wiimote_t *wiimote, int mesg_count, union cwiid_mesg mesg[], struct timespec *timestamp) {
	int i;

	//Datos iniciales de la llamada al callback
	printf("/--------------------------------------------------------\n");
	printf("|Callback de wiimote ID %i, mesg_count: %d\n", cwiid_get_id(wiimote), mesg_count);
	printf("|Time: %li (sec), %li (nsec)\n", timestamp->tv_sec, timestamp->tv_nsec);
	printf("|----------------------------------------\n");
	//Bucle que recorre todos los mensajes incluidos en la llamada al callback
	for (i = 0; i < mesg_count; i++) {
		printf("|>Mensaje nº %i, tipo: %i\n", i, mesg[i].type);
		printf("|>Estado-> tipo: %i, batería: %i, accesorio: %i\n", mesg[i].status_mesg.type, (int)(100.0*mesg[i].status_mesg.battery/CWIID_BATTERY_MAX), mesg[i].status_mesg.ext_type);
		printf("|>----------------------------------------\n");
	}
	printf("\\--------------------------------------------------------\n\n");
}


int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote[2] = {NULL, NULL}; //Array de gestores de Wiimote
	bdaddr_t bdaddr[2]; //Array de direcciones Bluetooth de Wiimotes
	int salir = 0; //Variable para interrumpir la ejecución
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth
	int i; //Índice del bucle
	int hayerror = 0; //Control de errores

	cwiid_set_err(error); //Asigno la función de error

	i = 0;
	while ((i < 2) && (hayerror == 0)) {
		printf("Poner el Wiimote nº %i en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n", i+1);
		getchar();
		printf("Buscando...\n");
		bdaddr[i] = *BDADDR_ANY; //Se aceptará la conexión de cualquier Wiimote
		if (!(wiimote[i] = cwiid_connect(&(bdaddr[i]), CWIID_FLAG_MESG_IFC))) { //Conexión y habilitación de envío de mensajes
			printf("No se pudo detectar Wiimote nº %i\n", i+1);
			hayerror = 1;
		}
		else {
			//Conexión establecida
			ba2str(&(bdaddr[i]), tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
			printf("Dirección Bluetooth del Wiimote nº %i (ID: %i): %s\n", i+1, cwiid_get_id(wiimote[i]), tmp);
		}
		i++;
	}

	//Una vez que se han conectado los Wiimotes, asigno los callback
	if (!hayerror) {
		for (i = 0; i < 2; i++) {
			if (cwiid_set_mesg_callback(wiimote[i], cwiid_callback)) { //Establezco el callback
				printf("No se pudo establecer el callback para el Wwiimote nº %i\n", i+1);
				hayerror = 1;
			}
			else {
				if (cwiid_set_rpt_mode(wiimote[i], CWIID_RPT_STATUS)) {
					printf("Error al establecer modo de informe en el Wiimote nº %i\n", i+1);
					hayerror = 1;
				}
				else {
					printf("El Wiimote nº %i con ID %i está dispuesto para ser usado\n", i+1, cwiid_get_id(wiimote[i]));
				}
			}
		}
	}
	
	//Si no hubo error, arranco la espera
	if (!hayerror) {
		//Al conectar y posteriormente activar callback de forma agrupada es posible que haya perdido el primer mensaje de estado de cada Wiimote
		//Vuelvo a solicitarlos aquí
		cwiid_command(wiimote[0], CWIID_CMD_STATUS, 0);
		cwiid_command(wiimote[1], CWIID_CMD_STATUS, 0);
		printf("Escribe x y pulsa enter para salir\n");
		while (!salir) {
			//Bucle principal, sólo espera
			if (getchar() == 'x') {salir = 1;}
		}
	}
	
	//Desconexión de Wiimotes
	for (i = 0; i < 2; i++) {
		if (wiimote[i] != NULL) { //Sólo desconectar los Wiimotes que hayan sido conectados
			if (cwiid_disconnect(wiimote[i])) {
				printf("Error al desconectar el Wiimote\n");
			}
		}
	}
	
	return 0;
}
